package be.cylab.java.roc;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class Roc {

    private List<Point> points;
    private int positive_examples_number;
    private int negative_examples_number;


    public Roc(double[] score, boolean[] true_alert) {
        if (score.length != true_alert.length) {
            throw new IllegalStateException(
                    "Score array and true alert array must be the same size");
        }
        for (int i = 0; i < score.length; i++) {
            Point point = new Point(score[i], true_alert[i]);
            this.points.add(point);
        }
        Collections.sort(points);
        positive_examples_number = Utils.countPositiveExamples(this.points);
        negative_examples_number = Utils.countNegativeExamples(this.points);
    }

    public List<RocCoordinates> computeRocPoints() {
        int true_positive = 0;
        int false_positive = 0;
        List<RocCoordinates> roc_points = new ArrayList<>();
        double previous_score = Double.NEGATIVE_INFINITY;
        for (Point point : this.points) {
            if (point.getScore() != previous_score) {
                double true_detection = true_positive / positive_examples_number;
                double false_alarm = false_positive /negative_examples_number;
                roc_points.add(new RocCoordinates(true_detection, false_alarm));
                previous_score = point.getScore();
            }
            if (point.getScore() == 1) {
                true_positive++;
            } else {
                false_positive++;
            }
        }
        double true_detection = true_positive / positive_examples_number;
        double false_alarm = false_positive /negative_examples_number;
        roc_points.add(new RocCoordinates(true_detection, false_alarm));
        return  roc_points;
    }


    public double computeAUC() {
        int false_positive = 0;
        int true_positive = 0;
        int previous_false_positive = 0;
        int previous_true_positive = 0;
        double area = 0;
        double previous_score = Double.NEGATIVE_INFINITY;
        for (Point point : this.points) {
            if (point.getScore() != previous_score) {
                area += this.trapezoidArea(
                        false_positive,
                        previous_false_positive,
                        true_positive,
                        previous_true_positive);
                previous_score = point.getScore();
                previous_false_positive = false_positive;
                previous_true_positive = true_positive;
            }
            if (point.isTrueAlert()) {
                true_positive++;
            } else {
                false_positive++;
            }
            area += this.trapezoidArea(
                    false_positive,
                    previous_false_positive,
                    true_positive,
                    previous_true_positive);
        }
        return (area / (positive_examples_number * negative_examples_number));
    }

    private double trapezoidArea(double x1, double x2, double y1, double y2) {
        double base = Math.abs(x1 - x2);
        double height_average = (y1 + y2) / 2;
        return (base * height_average);
    }

}
